(* 
 * Copyright (c) 2001, 2002 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Id: gontmodule.mli,v 1.7 2002/08/12 12:42:12 malekith Exp $
 *
 * Module interface: Gontmodule
 * Description: Manage simple Gont module system.
 *)

open Gonttree

(* Save given list of symbols and types as named module interface to given
 * file. *)
val save_module : out_channel -> (* module *) string -> symbol list ->
                  typedef list -> unit

(* Set module interface search path to given value. *)
val set_iface_search_path : string list -> unit

(* Reset module subsystem state. *)
val reset : unit -> unit

(* Return list of global symbols and types in given module. Used by
 * Gontcodegen to check whatever everything was implemented. *)
val globals : string -> symbol list * typedef list

(* Following functions are only used by Gontbind module. After binding
 * pass, we have symbols assigned in Ref, CrossModuleRef, NamedType and 
 * similar nodes, and we have no reason to use it. *)
 
(* Look for named symbol in named module. *)
val import_symbol : symbol_kind -> 
                    (* module *) string -> (* name *) string -> 
                    symbol

(* Look for named type in named module. *)
val import_type_symbol : (* module *) string -> (* name *) string -> typedef

(* Check whatever we can read module interface file, raise Error if not. *)
val open_module : string -> unit

(* Make named archive file from list of input files. It is conceptually
 * separate operation, i.e. it can be called without reset () and similar
 * things. *)
val make_archive : string -> string list -> unit

(* This structure is saved in addition to .o file. *)
type module_info = {mi_name: string;
                    mi_used_exns: string list;
                    mi_used_modules: string list;
                    mi_init_fnc: string;
                    mi_fini_fnc: string}

(* Write given module info to a file. Used at the end of codegeneration. *)
val save_module_info : out_channel -> module_info -> unit
val save_module_info_list : out_channel -> 
                            (* ldopts *) string -> module_info list -> unit

(* Retrive information about specific module. Used by linker. *)
val module_info_of_file : (* .go file name *) string -> module_info
val module_info_list_of_file : (* .ga file name *) string -> 
        ((* ldopts *) string * module_info list)
