/* 
 * Copyright (c) 2002 Michal Moskal <malekith@pld-linux.org>
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Id: c_array.c,v 1.2 2002/08/12 12:42:21 malekith Exp $
 *
 * Standard library for Gont, Array module C implementation.
 */

#include <gont/runtime.h>
#include <gont/array.h>

#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

typedef struct gont_array *t;

t gont_make_array(gont_ptroff sz)
{
	t ret;

	ret = gont_xmalloc0(sizeof(*ret));
	ret->data = gont_xmalloc0(sizeof(gont_ptroff) * sz);
	ret->size = sz;

	return ret;
}

t Array__make(gont_ptroff sz, gont_ptroff ini)
{
	t ret = gont_make_array(sz);

	if (ini != 0) {
		gont_ptroff i;

		for (i = 0; i < sz; i++)
			ret->data[i] = ini;
	}

	return ret;
}

t Array__init(gont_ptroff sz, struct gont_lambda_exp *ini)
{
	t ret = gont_make_array(sz);
	gont_ptroff i;

	for (i = 0; i < sz; i++) {
		ret->data[i] = (gont_ptroff)ini->fnc(ini->closure, i);
	}

	return ret;
}

gont_ptroff Array__length(t ar)
{
	return ar->size;
}

void Array__set(t ar, gont_ptroff idx, gont_ptroff val)
{
	if (idx >= ar->size)
		gont_invalid_argument("Array::set");
		
	ar->data[idx] = val;
}

gont_ptroff Array__get(t ar, gont_ptroff idx)
{
	if (idx >= ar->size)
		gont_invalid_argument("Array::get");
		
	return ar->data[idx];
}

/* TODO: calling this with newsz++ gets O(n^2) complexity, while we
 * can get O(nlogn). */
void Array__resize(t ar, gont_ptroff newsz, gont_ptroff ini)
{
	gont_ptroff *newdata = gont_xmalloc(newsz * sizeof(gont_ptroff));

	if (newsz > ar->size) {
		gont_ptroff i;
		
		memcpy(newdata, ar->data, ar->size * sizeof(gont_ptroff));
		
		for (i = ar->size; i < newsz; i++)
			newdata[i] = ini;
	} else {
		memcpy(newdata, ar->data, newsz * sizeof(gont_ptroff));
	}

	ar->size = newsz;
	ar->data = newdata;
}
